

' A property inspector control using a subclassed listbox.

PROGRAM	"proplistbox"
VERSION	"0.0001"

IMPORT "xst"
IMPORT "xsx"
IMPORT "gdi32"
IMPORT "user32"
IMPORT "kernel32"
IMPORT "comctl32"
IMPORT "comdlg32"

EXPORT

TYPE NMPROPLISTMODIFIED
	NMHDR	.hdr
	XLONG .curSel
END TYPE

TYPE VALUE
  XLONG .color
  STRING * 192 .str
  XLONG .num
END TYPE

TYPE FONT_CLASS
  STRING * 32 .name
  XLONG .size
  XLONG .bold
  XLONG .italic
  XLONG .underline
  XLONG .strikethrough
  XLONG .color
  XLONG .charset
END TYPE

TYPE PROPLIST_PROPERTY
  STRING * 192 .propName				' name of property
  STRING * 192 .curValue				' current value/string to display
  STRING * 192 .defValue				' default value/string
  XLONG .nItemType							' $$PIT_COMBO, $$PIT_EDIT, $$PIT_COLOR, $$PIT_FONT, $$PIT_FILE, $$PIT_NONE
  STRING * 192 .cmbItems				' string of combobox items, using | as separator
  STRING * 2 .cDataType   			' C - COLORREF, L - long, B - Boolean, S - String
  STRING * 32 .cat							' optional category string
  VALUE .value
	FONT_CLASS .font
  STRING * 192 .help						' optional help string
END TYPE

$$PROPLIST_MAXPROPERTIES = 128

DECLARE FUNCTION PropListBox ()
DECLARE FUNCTION PropList_NewComboItem (PROPLIST_PROPERTY this, name$, curValue$, defValue$, comboOptions$, dataType$, category$, help$)
DECLARE FUNCTION PropList_NewEditItem (PROPLIST_PROPERTY this, name$, curValue$, defValue$, dataType$, category$, help$)
DECLARE FUNCTION PropList_NewColorItem (PROPLIST_PROPERTY this, name$, color, defColor, category$, help$)
DECLARE FUNCTION PropList_NewFontItem (PROPLIST_PROPERTY this, name$, fontName$, defFontName$, category$, help$)
DECLARE FUNCTION PropList_NewFileItem (PROPLIST_PROPERTY this, name$, fileName$, defFileName$, category$, help$)
DECLARE FUNCTION PropList_GetProps (lpProp, @name$, @curValue$, @defValue$, @itemType, @comboItems$, @dataType$, @category$, @help$)
END EXPORT

DECLARE FUNCTION PropListProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION ListBoxProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION OnSelChange (fShow)
DECLARE FUNCTION MakeFontEx (sFont$, pointSize, bBold, bItalic, bUnderline, bStrikeThru)
DECLARE FUNCTION OnButtonClicked (hWnd)
DECLARE FUNCTION InvertLine (hWnd, x1, yFrom, x2, yTo)
DECLARE FUNCTION OpenFileDialog (hWnd, Caption$, Filespec$, InitialDir$, Filter$, DefExtension$, Flags)
DECLARE FUNCTION UpdatePropList (hWnd, PROPLIST_PROPERTY pItem[])
DECLARE FUNCTION EditProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION SendPropListNotifyMessage (hWnd, curSel)
DECLARE FUNCTION SetColor (txtColor, bkColor, wParam, lParam)

EXPORT
' class name
$$PROPLISTBOXCLASSNAME = "proplistboxctrl"

' custom messages
$$PROPLIST_SETPROPDATA 			= 1025	' assign PROPLIST_PROPERTY for listbox, wParam = index to item, lParm = address of PROPLIST_PROPERTY struct
$$PROPLIST_GETPROPDATA			= 1026	' get currently assigned PROPLIST_PROPERTY at index, lParam = index, return = pointer to property struct
$$PROPLIST_GETCURSEL				= 1027	' returns the current selection index
$$PROPLIST_CLEARPROPDATA    = 1028  ' deletes all current property items
$$PROPLIST_SELECTION_CHANGE = 1029  ' notify message sent to parent control when a selection change has occurred

' Edit box return message
$$EDITBOX_RETURN = 402

' CONST PIT = property item type, Button is default
$$PIT_COMBO   = 1
$$PIT_EDIT    = 2
$$PIT_COLOR   = 3
$$PIT_FONT    = 4
$$PIT_FILE    = 5
$$PIT_NONE    = 0

END EXPORT

' Inspector window IDs
$$IDC_LISTBOX 	= 1200
$$IDC_COMBO   	= 1201
$$IDC_BUTTON  	= 1202
$$IDC_EDIT    	= 1203
$$IDC_LABEL1    = 1204
$$IDC_LABEL2    = 1205

FUNCTION PropListBox ()

	WNDCLASS wc
	STATIC init
	SHARED hInst
	SHARED hCursorArrow, hCursorSize
	SHARED hFont, hComboFont, hFontBold

' do this once
	IF init THEN RETURN
	init = $$TRUE

' get Instance handle
	hInst = GetModuleHandleA (0)

	hCursorArrow = LoadCursorA (NULL, $$IDC_ARROW)

' Use a cursor from resource file.
  hCursorSize = LoadCursorA (GetModuleHandleA (&"proplistbox.dll"), &"hsplit")
	IFZ hCursorSize THEN
		XstGetCommandLineArguments (@argCount, @argv$[])
		prog$ = argv$[0]
		hCursorSize = LoadCursorA (GetModuleHandleA (&prog$), &"hsplit")
	END IF

' If we couldn't get the horizontal splitter (custom) cursor then use arrow
  IFZ hCursorSize THEN
    hCursorSize = hCursorArrow
  END IF

	hFont 		 = MakeFontEx ("MS Sans Serif", 8, $$FALSE, $$FALSE, $$FALSE, $$FALSE)
	hComboFont = MakeFontEx ("MS Sans Serif", 6, $$FALSE, $$FALSE, $$FALSE, $$FALSE)
	hFontBold	 = MakeFontEx ("MS Sans Serif", 6, $$TRUE, $$FALSE, $$FALSE, $$FALSE)

' Register control window class.
	className$       = "proplistboxctrl"
	wc.style         = $$CS_HREDRAW OR $$CS_VREDRAW OR $$CS_GLOBALCLASS
  wc.lpfnWndProc   = &PropListProc()
  wc.cbClsExtra    = 0
  wc.cbWndExtra    = 0
  wc.hInstance     = hInst
  wc.hIcon         = NULL
  wc.hCursor       = LoadCursorA (0, $$IDC_ARROW)
  wc.hbrBackground = ($$COLOR_BTNFACE + 1) '$$COLOR_WINDOW + 1
  wc.lpszMenuName  = NULL
  wc.lpszClassName = &className$

  RETURN RegisterClassA (&wc)

END FUNCTION

FUNCTION PropListProc (hWnd, msg, wParam, lParam)

	MINMAXINFO mmi
	DRAWITEMSTRUCT dis
	MEASUREITEMSTRUCT mis
	RECT rc, rc1, rectFull, rc2, rcc

	SHARED hInst
	SHARED hFont, hComboFont, hFontBold

	SHARED ghList
	SHARED ghButton, ghCombo, ghEdit
	SHARED ghLabel2, ghLabel1

	SHARED bTracking, nDivider, bDivIsSet, curSel, prevSel
	SHARED xMouse

	SHARED lpOldListProc, lpOldEditProc
	SHARED hCursorArrow, hCursorSize

	SHARED PROPLIST_PROPERTY pItem[]

	$LOFFSET = 4

	SELECT CASE msg

		CASE $$PROPLIST_CLEARPROPDATA:
			DIM pItem[$$PROPLIST_MAXPROPERTIES]
			curSel = -1
			SetWindowTextA (GetDlgItem (hWnd, $$IDC_LABEL2), NULL)
			SetWindowTextA (GetDlgItem (hWnd, $$IDC_LABEL1), &"Help")
			UpdatePropList (hWnd, @pItem[])

		CASE $$PROPLIST_GETCURSEL:
			RETURN curSel

		CASE $$PROPLIST_GETPROPDATA:
			IF lParam > $$PROPLIST_MAXPROPERTIES THEN RETURN
			RETURN &pItem[lParam]

		CASE $$WM_DESTROY:
			DeleteObject (hFont)
			DeleteObject (hComboFont)
			DeleteObject (hFontBold)
			DIM pItem[]

		CASE $$PROPLIST_SETPROPDATA :
			IFZ lParam THEN RETURN
			IF wParam > $$PROPLIST_MAXPROPERTIES THEN RETURN
			XstCopyMemory (lParam, &pItem[wParam], SIZE(PROPLIST_PROPERTY))
			UpdatePropList (hWnd, @pItem[])

		CASE $$WM_CREATE

			DIM pItem[$$PROPLIST_MAXPROPERTIES]

			bTracking = $$FALSE
			nDivider 	= 0
			bDivIsSet = $$FALSE
			curSel 		= -1

			GetClientRect (hWnd, &rc)

			' Create the OwnerDraw List box for our control properties...
			ghList = CreateWindowExA ($$WS_EX_CLIENTEDGE, &"LISTBOX", &"", _
			$$WS_CHILD | $$WS_VISIBLE | _
			$$WS_TABSTOP | $$WS_BORDER | _
			$$LBS_HASSTRINGS | _
			$$LBS_OWNERDRAWFIXED | $$LBS_NOINTEGRALHEIGHT | _
			$$LBS_NOTIFY | _
			$$WS_VSCROLL, _
			0, 0, rc.right, rc.bottom, _
			hWnd, _
			$$IDC_LISTBOX, _
			hInst, NULL)

			' SubClass the ListBox
			lpOldListProc = SetWindowLongA (ghList, $$GWL_WNDPROC, &ListBoxProc ())

			SendMessageA (ghList, $$WM_SETFONT, hFont, 0)

			' Create the Help Label at bottom of Properties Listbox
			ghLabel2 = CreateWindowExA (0, &"STATIC", &"This is the help", _
			$$WS_CHILD | $$WS_VISIBLE | $$WS_CLIPSIBLINGS | $$SS_NOTIFY, _
			6, rc.bottom - 100, rc.right, 66, _
			hWnd, _
			$$IDC_LABEL2, _
			ghInst, NULL)
			SendMessageA (ghLabel2, $$WM_SETFONT, hFont, 0)
			SetWindowTextA (ghLabel2, &"More Later.  ")

			' Create the Bold Help Label at bottom of Properties Listbox
			ghLabel1 = CreateWindowExA ($$WS_EX_CLIENTEDGE, &"STATIC", &"Help", _
			$$WS_CHILD | $$WS_VISIBLE | $$WS_CLIPSIBLINGS | $$SS_NOTIFY, _
			6, rc.bottom - 100, rc.right, 66, _
			hWnd, _
			$$IDC_LABEL1, _
			ghInst, NULL)
			SendMessageA (ghLabel1, $$WM_SETFONT, hFontBold, 0)

			' Create the controls that we'll need to edit the Properties with.
			' They are all hidden for now and resized later.
			ghButton = CreateWindowExA (0, &"BUTTON", &"...", _
			$$WS_CHILD | $$WS_CLIPSIBLINGS | _
			$$BS_PUSHBUTTON, _
			0, 0, 8, 12, _
			ghList, _
			$$IDC_BUTTON, _
			hInst, NULL)

			SendMessageA (ghButton, $$WM_SETFONT, hFont, 0)

			ghCombo = CreateWindowExA (0, &"COMBOBOX", &"", _
			$$WS_CHILD | _
			$$CBS_DROPDOWNLIST _
			| $$CBS_NOINTEGRALHEIGHT, _
			0, 0, 10, 100, _
			ghList, _
			$$IDC_COMBO, _
			hInst, NULL)

			SendMessageA (ghCombo, $$WM_SETFONT, hComboFont, 0)

			ghEdit = CreateWindowExA (0, &"EDIT", &"TRUE", _
			$$WS_CHILD | $$WS_CLIPSIBLINGS | _
			$$ES_LEFT | $$ES_AUTOHSCROLL, _
			0, 0, 10, 20, _
			ghList, _
			$$IDC_EDIT, _
			hInst, NULL)

			lpOldEditProc = SetWindowLongA (ghEdit, $$GWL_WNDPROC, &EditProc())

			SendMessageA (ghEdit, $$EM_LIMITTEXT, 255, 0)
			SendMessageA (ghEdit, $$WM_SETFONT, hFont, 0)
			RETURN 1

		CASE $$WM_CTLCOLORSTATIC :
			RETURN SetColor (0, GetSysColor($$COLOR_BTNFACE), wParam, lParam)

		CASE $$WM_SYSCOMMAND
			SELECT CASE wParam
				CASE $$SC_MAXIMIZE, $$SC_ZOOM, $$SC_MINIMIZE, $$SC_ICON : RETURN
			END SELECT

		CASE $$WM_LBUTTONDBLCLK, $$WM_NCLBUTTONDBLCLK :
			RETURN

		CASE $$WM_MEASUREITEM
			XstCopyMemory (lParam, &mis, SIZE (mis))
			' arbitrary number for height
			mis.itemHeight = 18
			XstCopyMemory (&mis, lParam, SIZE (mis))
			RETURN 1

		CASE $$WM_DRAWITEM
			XstCopyMemory (lParam, &dis, SIZE (dis))
			rc        = dis.rcItem
			IF nDivider = 0 THEN nDivider = (rc.right - rc.left-10)  / 2
			rc.left   = nDivider
			rc2       = dis.rcItem
			rc2.right = rc.left - 1
			nIndex    = dis.itemID
			nAction   = dis.itemAction
			IF nAction = $$ODA_SELECT || nAction = $$ODA_DRAWENTIRE THEN
        'If the Item is selected, then setup the HilightColors
        IF dis.itemState & $$ODS_SELECTED THEN
          IF bTracking = $$TRUE THEN
            'Prevent any highlighting
            curSel = -1
            SetTextColor (dis.hDC, GetSysColor ($$COLOR_WINDOWTEXT))
            SetBkColor (dis.hDC, GetSysColor ($$COLOR_WINDOW))
          ELSE
            curSel = dis.itemID
            SetTextColor (dis.hDC, GetSysColor ($$COLOR_HIGHLIGHTTEXT))
            SetBkColor (dis.hDC, GetSysColor ($$COLOR_HIGHLIGHT))
          END IF
        ELSE
          SetTextColor (dis.hDC, GetSysColor($$COLOR_WINDOWTEXT))
          SetBkColor (dis.hDC, GetSysColor($$COLOR_WINDOW))
        END IF
        ' ExTextOut does the same work as FillRect with the ETO_OPAQUE Flag set.
        ' No extra BrushHandle is needed here.
				' write the Property Name in the first rectangle, left side
				name$ = pItem[nIndex].propName
        ExtTextOutA (dis.hDC, rc2.left+$LOFFSET, rc2.top+2, $$ETO_OPAQUE, &rc2, &name$, LEN(name$), 0)
        ' If the Property is a Color, then draw a colored Rectangle before the RGB(x,x,x)
        IF INSTR (LCASE$(name$), "color") THEN
          ' First draw out the Text and BackColor
					value$ = pItem[nIndex].curValue
          ExtTextOutA (dis.hDC, rc.left+20+$LOFFSET, rc.top+2, $$ETO_OPAQUE, &rc, &value$, LEN(value$), 0)
          ' Now setup and paint the colored Rectangle
          rcc.left = rc.left + $LOFFSET
          rcc.top = rc.top + 2
          rcc.right = rc.left + 13 + $LOFFSET
          rcc.bottom = rc.top + 14
          hBrush = CreateSolidBrush (XLONG(value$))
          FillRect (dis.hDC, &rcc, hBrush)
          DeleteObject (hBrush)
        ELSE
					value$ = pItem[nIndex].curValue
          ExtTextOutA (dis.hDC, rc.left+$LOFFSET, rc.top+2, $$ETO_OPAQUE, &rc, &value$, LEN(value$), 0)
        END IF
        DrawEdge (dis.hDC, &rc2, $$EDGE_SUNKEN, $$BF_BOTTOMRIGHT)
        DrawEdge (dis.hDC, &rc,  $$EDGE_SUNKEN, $$BF_BOTTOM)
      END IF
      RETURN 1

		CASE $$WM_GETMINMAXINFO
			XstCopyMemory (lParam, &mmi, SIZE (mmi))
			mmi.ptMinTrackSize.x = 80
			mmi.ptMinTrackSize.y = 90
			XstCopyMemory (&mmi, lParam, SIZE (mmi))

		CASE $$WM_SIZE, $$WM_MOVE :
'			w = LOWORD (lParam)
'			h = HIWORD (lParam)
			GetClientRect (hWnd, &rc)
			MoveWindow (GetDlgItem (hWnd, $$IDC_LISTBOX), 0, 0, rc.right, rc.bottom - 90, 1)
			MoveWindow (GetDlgItem (hWnd, $$IDC_LABEL2), 6, rc.bottom - 63, rc.right - 12, 61, 1)
			MoveWindow (GetDlgItem (hWnd, $$IDC_LABEL1), 0, rc.bottom - 83, rc.right, 83, 1)

			IF IsWindowVisible (hWnd) THEN
				SetWindowPos (hWnd, $$HWND_TOP, 0, 0, 0, 0, _
					$$SWP_FRAMECHANGED | $$SWP_NOCOPYBITS | $$SWP_NOMOVE | _
					$$SWP_NOSIZE | $$SWP_SHOWWINDOW | $$SWP_NOREPOSITION)
			END IF

		CASE $$WM_SETFOCUS
			SetFocus (GetDlgItem(hWnd, $$IDC_LISTBOX))

		CASE $$WM_COMMAND
			' Properties Inspector ListBox control notifications...
			SELECT CASE LOWORD (wParam)
				CASE $$IDC_LISTBOX
					SELECT CASE HIWORD (wParam)
						CASE $$LBN_SELCHANGE
							SetWindowTextA (GetDlgItem(hWnd, $$IDC_LABEL2), &"More Later.   ")
							IF xMouse > nDivider+5 THEN
								OnSelChange ($$TRUE)
							ELSE
								OnSelChange ($$FALSE)
							END IF
					END SELECT
			END SELECT

		CASE $$WM_MOUSEMOVE, $$WM_NCMOUSEMOVE
			SetFocus (hWnd)
			RETURN

	END SELECT

	RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

END FUNCTION

FUNCTION ListBoxProc (hWnd, msg, wParam, lParam)

	STATIC POINT pt
	RECT rc
	SHARED ghCombo, ghEdit, ghButton, ghList
	SHARED PROPLIST_PROPERTY pItem[]

	SHARED nDivider, bTracking
	SHARED nOldDivX, nDivTop, nDivBtm
	SHARED lpOldListProc
	SHARED hCursorSize, hCursorArrow
	SHARED curSel
	SHARED xMouse

	SELECT CASE msg

		CASE $$WM_VSCROLL
			OnSelChange ($$FALSE)

		CASE $$WM_COMMAND
			SELECT CASE LOWORD (wParam)
				CASE $$IDC_COMBO
					SELECT CASE HIWORD (wParam)
						CASE $$CBN_SELCHANGE
							selStr$ = NULL$ (255)
							idx = SendMessageA (ghCombo, $$CB_GETCURSEL, 0, 0)
							SendMessageA (ghCombo, $$CB_GETLBTEXT, idx, &selStr$)
							selStr$ = CSIZE$ (selStr$)
							IF pItem[curSel].curValue = selStr$ THEN RETURN
							pItem[curSel].curValue = selStr$

							ShowWindow (ghCombo, $$SW_HIDE)

							' send a notify message to parent window
							SendPropListNotifyMessage (hWnd, curSel)
					END SELECT

				CASE $$IDC_EDIT
					SELECT CASE HIWORD (wParam)

						CASE $$EDITBOX_RETURN
							SetFocus (GetParent(hWnd))

						CASE $$EN_SETFOCUS
							SendMessageA (lParam, $$EM_SETSEL, 0, -1)
						CASE $$EN_KILLFOCUS		'EN_CHANGE
							tLen = SendMessageA (ghEdit, $$WM_GETTEXTLENGTH, 0, 0) + 1
							newStr$ = NULL$ (tLen)
							SendMessageA (ghEdit, $$WM_GETTEXT, tLen, &newStr$)
							newStr$ = CSIZE$ (newStr$)
							IF pItem[curSel].curValue = newStr$ THEN RETURN
							pItem[curSel].curValue = newStr$

							' send a notify message to parent window
							SendPropListNotifyMessage (hWnd, curSel)
					END SELECT

				CASE $$IDC_BUTTON
					SELECT CASE HIWORD (wParam)
						CASE $$BN_CLICKED
							OnButtonClicked (hWnd)
					END SELECT
			END SELECT

		CASE $$WM_LBUTTONDOWN
			IF IsWindowVisible (ghCombo)  THEN ShowWindow (ghCombo, $$SW_HIDE)
			IF IsWindowVisible (ghEdit)   THEN ShowWindow (ghEdit, $$SW_HIDE)
			IF IsWindowVisible (ghButton) THEN ShowWindow (ghButton, $$SW_HIDE)

			' -----------------------[ Splitter ]-----------------------
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			xMouse = pt.x

			IF ((pt.x >= nDivider - 5) && (pt.x <= nDivider + 5)) THEN
				' if mouse clicked on divider line, then start resizing
				SetCursor (hCursorSize)
				GetWindowRect (hWnd, &rc)
				rc.left = rc.left + 10
				rc.right = rc.right - 20

				' do not let mouse leave the list box boundary
				ClipCursor (&rc)
				GetClientRect (hWnd, &rc)
				bTracking = $$TRUE

				nDivTop = rc.top
				nDivBtm = rc.bottom
				nOldDivX = pt.x

				InvertLine (hWnd, nOldDivX, nDivTop, nOldDivX, nDivBtm)

				' capture the mouse
				SetCapture (hWnd)
			END IF

		CASE $$WM_MOUSEMOVE
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			xMouse = pt.x

			' move divider line to the mouse position
			' if columns are currently being resized
			IF bTracking = $$TRUE THEN
				' remove old divider line
				InvertLine (hWnd, nOldDivX, nDivTop, nOldDivX, nDivBtm)
				' draw new divider line
				InvertLine (hWnd, pt.x, nDivTop, pt.x, nDivBtm)
				nOldDivX = pt.x
			ELSE
				' set the cursor to a sizing cursor if
				' the cursor is over the row divider
				IF (pt.x >= nDivider - 5) && (pt.x <= nDivider + 5) THEN
					SetCursor (hCursorSize)
				END IF
			END IF

		CASE $$WM_LBUTTONUP
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			xMouse = pt.x
			' if columns were being resized then this indicates
			' that mouse is up so resizing is done.  Need to redraw
			' columns to reflect their new widths.
			IF bTracking = $$TRUE THEN
				bTracking = $$FALSE
				ReleaseCapture ()
				ClipCursor (NULL)
				InvertLine (hWnd, pt.x, nDivTop, pt.x, nDivBtm)
				' set the divider position to the new value
				nDivider = pt.x
				' Self paint the ListBox control...
				InvalidateRect (hWnd, NULL, 1)
				UpdateWindow (hWnd)
			END IF

		CASE $$WM_CAPTURECHANGED
			IF lParam <> ghList THEN
				IF bTracking = $$TRUE THEN
					bTracking = $$FALSE
					ReleaseCapture ()
					ClipCursor (NULL)
					InvalidateRect (ghList, NULL, 1)
					UpdateWindow (ghList)
				END IF
			END IF
			' ---------------------[ end of splitter ]------------------

	END SELECT

	RETURN CallWindowProcA (lpOldListProc, hWnd, msg, wParam, lParam)

END FUNCTION

FUNCTION OnSelChange (fShow)

	RECT rc
	SHARED ghList, ghCombo, ghButton, ghEdit
	SHARED ghLabel1, ghLabel2
	SHARED PROPLIST_PROPERTY pItem[]

	SHARED nDivider
	SHARED curSel, prevSel
	SHARED nLastBox

	curSel = SendMessageA (ghList, $$LB_GETCURSEL, 0, 0)
	IF curSel = -1 THEN RETURN
	SendMessageA (ghList, $$LB_GETITEMRECT, curSel, &rc)
	prevSel = curSel
	rc.left = nDivider

  IF IsWindowVisible (ghCombo)  THEN ShowWindow (ghCombo,  $$SW_HIDE)
  IF IsWindowVisible (ghButton) THEN ShowWindow (ghButton, $$SW_HIDE)
  IF IsWindowVisible (ghEdit)   THEN ShowWindow (ghEdit,   $$SW_HIDE)

	' set help messages
	SetWindowTextA (ghLabel1, &pItem[curSel].propName)
	SetWindowTextA (ghLabel2, &pItem[curSel].help)

	' Exit here if the user clicked on the left side of the Proplist
	IFZ fShow THEN RETURN

	IF pItem[curSel].nItemType = $$PIT_NONE THEN RETURN

	IF pItem[curSel].nItemType = $$PIT_COMBO THEN

		' display the combo box and move it to the new location
		nLastBox = 0

		IF IsWindow (ghCombo) THEN
			MoveWindow (ghCombo, rc.left-1, rc.top-2, rc.right+2 - rc.left, rc.bottom + 100, 1)
		END IF

		' add the choices for this particular property
		cmbItems$ = pItem[curSel].cmbItems
		lBoxSelText$ = pItem[curSel].curValue
		SendMessageA (ghCombo, $$CB_RESETCONTENT, 0, 0)

		delimiter$ = "|"
		nCount = XstTally (cmbItems$, delimiter$) + 1

		FOR i = 1 TO nCount
			dataItem$ = XstParse$ (cmbItems$, "|", i)
			' add each string to the ComboBox
			SendMessageA (ghCombo, $$CB_ADDSTRING, i - 1, &dataItem$)
		NEXT i

		ShowWindow (ghCombo, $$SW_SHOW)
		SetFocus (ghCombo)

		' jump to the property's current value in the combo box
		j = SendMessageA (ghCombo, $$CB_FINDSTRINGEXACT, 0, &lBoxSelText$)
		IF j <> $$CB_ERR THEN
			SendMessageA (ghCombo, $$CB_SETCURSEL, j, 0)
		ELSE
			' there is no current value, so default to first in list
			SendMessageA (ghCombo, $$CB_SETCURSEL, 0, 0)
		END IF

	ELSE
		IF pItem[curSel].nItemType = $$PIT_EDIT THEN

			' display edit box
			nLastBox = 1
			prevSel = curSel

			IF IsWindow (ghEdit) THEN
				MoveWindow (ghEdit, rc.left, rc.top, rc.right - rc.left, (rc.bottom - rc.top)-2, 1)
			END IF

			lBoxSelText$ = pItem[curSel].curValue
			ShowWindow (ghEdit, $$SW_SHOW)
			' set the text in the edit box to the property's current value
			SendMessageA (ghEdit, $$WM_SETTEXT, 0, &lBoxSelText$)
			SetFocus (ghEdit)

		ELSE
			' displays a button if the property is a Color/File/Font chooser
			nLastBox = 2
			prevSel = curSel

			nWidth = rc.right - rc.left

			IF nWidth > 25 THEN
				rc.left = rc.right - 26
			END IF

			rc.bottom = rc.bottom - 3

			IF IsWindow (ghButton) THEN
				MoveWindow (ghButton, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, 1)
			END IF

			ShowWindow (ghButton, $$SW_SHOW)
			SetFocus (ghButton)
		END IF
	END IF
END FUNCTION

FUNCTION MakeFontEx (sFont$, pointSize, bBold, bItalic, bUnderline, bStrikeThru)

' Create a font and return the handle
    hDC = GetDC ($$HWND_DESKTOP)
    cyPixels  = GetDeviceCaps (hDC, $$LOGPIXELSY)
    ReleaseDC ($$HWND_DESKTOP, hDC)
    charSet = 0
    pointSize = 0 - (pointSize * cyPixels) / 72
		IF bBold THEN weight = $$FW_BOLD ELSE weight = $$FW_NORMAL
    RETURN CreateFontA (pointSize, 0, 0, 0, weight, bItalic, bUnderline, bStrikeThru, charSet, $$OUT_TT_PRECIS, $$CLIP_DEFAULT_PRECIS, $$DEFAULT_QUALITY, $$FF_DONTCARE, &sFont$)

END FUNCTION

FUNCTION OnButtonClicked (hWnd)

	SHARED PROPLIST_PROPERTY pItem[]
	CHOOSECOLOR ColorSpec
	SHARED ghList
	SHARED curSel
	CHOOSEFONT cf
	LOGFONT lfFont
	SHARED ghWndInspector
	STATIC ULONG ccBuffer[]
	STATIC init

	' display the appropriate common dialog depending on what type
	' of chooser is associated with the property

	SELECT CASE pItem[curSel].nItemType

		CASE $$PIT_COLOR			' First check for the Choose Color dialog...

			currClr$ = pItem[curSel].curValue + CHR$ (0)

			IF currClr$ <> "" THEN
				initClr = XLONG (currClr$)
			ELSE
				' use a default instead
				initClr = RGB (0xFF, 0x80, 0x80)
			END IF

			' fill custom color array with white color values
			IFZ init THEN
				DIM ccBuffer[15]
				FOR i = 0 TO 15
					ccBuffer[i] = 0xFFFFFF
				NEXT i
				init = $$TRUE
			END IF

			ColorSpec.lStructSize 	= SIZE (ColorSpec)
			ColorSpec.hwndOwner 		= ghList
			ColorSpec.lpCustColors 	= &cColor
			ColorSpec.rgbResult 		= initClr
			ColorSpec.flags 				= ColorSpec.flags OR $$CC_RGBINIT
			ColorSpec.lpCustColors 	= &ccBuffer[0]
			' open choosecolor dialog
			ret = ChooseColorA (&ColorSpec)

			IF ret THEN 		' check if user cancelled dialog ?
				clrStr$ = HEXX$(ColorSpec.rgbResult, 6)
				pItem[curSel].curValue = clrStr$
				InvalidateRect (ghList, NULL, 1)
				UpdateWindow (ghList)

				' send a notify message to parent window
				SendPropListNotifyMessage (hWnd, curSel)
			END IF

		CASE $$PIT_FILE				' Next check for the Open File Dialog...

			zTxt$ = "All pictures  (*.bmp,*.ico)|*.BMP;*.ICO|"
			zTxt$ = zTxt$ + "Bitmap (*.bmp)|*.BMP|"
			zTxt$ = zTxt$ + "Icon   (*.ico)|*.ICO|"

			currPath$ = pItem[curSel].curValue

			IF currPath$ = "none" THEN
				fName$ = "*.ico;*.bmp"
			ELSE
				fName$ = currPath$
			END IF

			style = $$OFN_EXPLORER OR $$OFN_FILEMUSTEXIST OR $$OFN_HIDEREADONLY
			ret = OpenFileDialog (hWnd, "Select File", @fName$, "", zTxt$, "", style)

			IF ret THEN
				IF fName$ <> "" THEN
					pItem[curSel].curValue = fName$
					InvalidateRect (ghList, NULL, 1)
					UpdateWindow (ghList)

					' send a notify message to parent window
					SendPropListNotifyMessage (hWnd, curSel)
				END IF
			END IF


		CASE $$PIT_FONT				' Last, check for the ChooseFont() dialog...

			hDC = CreateDCA (&"DISPLAY", NULL, NULL, NULL)
			lfFont.height 		= -MulDiv (pItem[curSel].font.size, GetDeviceCaps (hDC, $$LOGPIXELSY), 72)
			DeleteDC (hDC)
			lfFont.weight 		= pItem[curSel].font.bold
			lfFont.italic 		= pItem[curSel].font.italic
			lfFont.underline 	= pItem[curSel].font.underline
			lfFont.faceName 	= pItem[curSel].font.name
			cf.lStructSize 		= SIZE (cf)
			cf.hwndOwner 			= ghWndInspector		'ghMain
			cf.lpLogFont 			= &lfFont
			cf.flags 					= $$CF_EFFECTS OR $$CF_SCREENFONTS OR $$CF_INITTOLOGFONTSTRUCT
			ret = ChooseFontA (&cf)

			IF ret THEN		' check if user cancelled dialog ?
				faceName$ = lfFont.faceName
				pItem[curSel].curValue 				= lfFont.faceName
				pItem[curSel].value.str 			= lfFont.faceName
				pItem[curSel].font.size      	= cf.iPointSize/10
				pItem[curSel].font.bold      	= lfFont.weight
				pItem[curSel].font.italic    	= lfFont.italic
				pItem[curSel].font.underline 	= lfFont.underline
				pItem[curSel].font.name      	= lfFont.faceName

				InvalidateRect (ghList, NULL, 1)
				UpdateWindow (ghList)

				' send a notify message to parent window
				SendPropListNotifyMessage (hWnd, curSel)
			END IF
	END SELECT

END FUNCTION

FUNCTION InvertLine (hWnd, x1, yFrom, x2, yTo)

	POINT oldpt

	' Get DC of ListBox
	hDC = GetDC (hWnd)

	' Use the default Pen color and style...
	' Set the GDI foreground mix mode
	nDrawMode = SetROP2 (hDC, $$R2_NOT)

	' Draw the Line
	MoveToEx (hDC,x1, yFrom, &oldpt)
	LineTo (hDC, x2, yTo)

	' Restore DC
	SetROP2 (hDC, nDrawMode)
	ReleaseDC (hWnd, hDC)

END FUNCTION

FUNCTION OpenFileDialog (hWnd, Caption$, Filespec$, InitialDir$, Filter$, DefExtension$, Flags)

	SHARED CURDIR$
	OPENFILENAME ofn
	SHARED hInst

	SHARED ghWndInspector, ghWndTool, ghWndEditor

	ShowWindow	(ghWndInspector, $$SW_HIDE)
	ShowWindow	(ghWndTool, $$SW_HIDE)
	ShowWindow	(ghWndEditor, $$SW_HIDE)

	XstReplace (@Filter$, "|", CHR$ (0), 0)

	IFZ InitialDir$ THEN
		XstGetCurrentDirectory (@CURDIR$)
		InitialDir$ = CURDIR$
	END IF

	IFZ Filespec$ THEN
		Filespec$ = NULL$ (255)
	ELSE
		Filespec$ = Filespec$ + NULL$ (255 - LEN (Filespec$))
	END IF

	FileTitle$ = NULL$ (255)

	ofn.lStructSize = SIZE (ofn)
	ofn.hwndOwner = hWnd
	ofn.hInstance = hInst
	ofn.lpstrFilter = &Filter$
	ofn.nFilterIndex = 1
	ofn.lpstrFile = &Filespec$
	ofn.nMaxFile = LEN (Filespec$)
	ofn.lpstrFileTitle = &FileTitle$
	ofn.nMaxFileTitle = LEN (FileTitle$)
	ofn.lpstrInitialDir = &InitialDir$
	IF Caption$ THEN
		ofn.lpstrTitle = &Caption$
	END IF
	ofn.flags = Flags
	ofn.lpstrDefExt = &DefExtension$

	' call dialog
	IFZ GetOpenFileNameA (&ofn) THEN
		Flags = ofn.flags
		Filespec$ = ""
		ret = 0
	ELSE
		Flags = ofn.flags
		Filespec$ = CSIZE$ (Filespec$)
		ret = $$TRUE
	END IF

	ShowWindow (ghWndInspector, $$SW_SHOW)
	ShowWindow (ghWndTool, $$SW_SHOW)
	ShowWindow (ghWndEditor, $$SW_SHOW)

	RETURN ret

END FUNCTION

FUNCTION UpdatePropList (hWnd, PROPLIST_PROPERTY pItem[])

	hList = GetDlgItem (hWnd, $$IDC_LISTBOX)
	hButton = GetDlgItem (hWnd, $$IDC_BUTTON)
	hEdit = GetDlgItem (hWnd, $$IDC_EDIT)
	hCombo = GetDlgItem (hWnd, $$IDC_COMBO)

	ShowWindow (hCombo, $$SW_HIDE)
	ShowWindow (hButton, $$SW_HIDE)
	ShowWindow (hEdit, $$SW_HIDE)

	IFZ pItem[] THEN RETURN

	' Load the properties inspector
	LockWindowUpdate (hWnd)

	SendMessageA (hList, $$LB_RESETCONTENT, 0, 0)
	upp = UBOUND(pItem[])
	FOR i = 0 TO upp
		IF pItem[i].nItemType = $$PIT_NONE THEN DO NEXT
		SendMessageA (hList, $$LB_ADDSTRING, 0, &pItem[i].propName)
	NEXT i

	LockWindowUpdate (NULL)

	InvalidateRect (hList, NULL, 0)
	UpdateWindow (hList)

END FUNCTION

'
' ###################################
' #####  PropList_NewComboItem  #####
' ###################################
'
'
'
FUNCTION PropList_NewComboItem (PROPLIST_PROPERTY this, name$, curValue$, defValue$, comboOptions$, dataType$, category$, help$)
	this.value.num   = 0
	this.value.color = 0
  this.value.str = ""
  this.propName  = name$
  this.curValue  = curValue$
  this.defValue  = defValue$
  this.nItemType = $$PIT_COMBO
  this.cmbItems  = comboOptions$   	' eg, "0 - None|1 - Align Top|2 - Align Bottom|3 - Align Left|4 - Align Right"
  this.cDataType = dataType$    		' C - COLORREF, L - long, B - Boolean, S - String
  this.cat       = category$
  this.help      = help$

END FUNCTION

FUNCTION PropList_NewEditItem (PROPLIST_PROPERTY this, name$, curValue$, defValue$, dataType$, category$, help$)
	this.value.num   = 0
	this.value.color = 0
  this.value.str = curValue$
  this.propName  = name$
  this.curValue  = curValue$
  this.defValue  = defValue$
  this.nItemType = $$PIT_EDIT
  this.cmbItems  = ""
  this.cDataType = dataType$
  this.cat       = category$
  this.help      = help$

END FUNCTION

FUNCTION PropList_NewColorItem (PROPLIST_PROPERTY this, name$, color, defColor, category$, help$)
	this.value.str   = ""
	this.value.num   = 0
	this.value.color = color
  this.propName  = name$
  this.curValue  = HEXX$(color)
  this.defValue  = HEXX$(defColor)
  this.nItemType = $$PIT_COLOR
  this.cmbItems  = ""
  this.cDataType = "C"
  this.cat       = category$
  this.help      = help$

END FUNCTION
'
' ##################################
' #####  PropList_NewFontItem  #####
' ##################################
'
'
'
FUNCTION PropList_NewFontItem (PROPLIST_PROPERTY this, name$, fontName$, defFontName$, category$, help$)
	this.value.num   = 0
	this.value.color = 0
	this.value.str = fontName$
  this.propName  = name$
  this.curValue  = fontName$
  this.defValue  = defFontName$
  this.nItemType = $$PIT_FONT
  this.cmbItems  = ""
  this.cDataType = "S"
  this.cat       = category$
  this.help      = help$

END FUNCTION
'
' ##################################
' #####  PropList_NewFileItem  #####
' ##################################
'
'
'
FUNCTION PropList_NewFileItem (PROPLIST_PROPERTY this, name$, fileName$, defFileName$, category$, help$)
	this.value.num   = 0
	this.value.color = 0
	this.value.str = fileName$
	this.propName  = name$
  this.curValue  = fileName$
  this.defValue  = defFileName$
  this.nItemType = $$PIT_FILE
  this.cmbItems  = ""
  this.cDataType = "S"
  this.cat       = category$
  this.help      = help$

END FUNCTION

FUNCTION EditProc (hWnd, msg, wParam, lParam)

	SHARED lpOldEditProc
	SELECT CASE msg

		CASE $$WM_KEYDOWN :			' WM_KEYDOWN returns virtKey constants
			virtKey = wParam
			IF virtKey = $$VK_RETURN THEN
				id = GetWindowLongA (hWnd, $$GWL_ID)
				wParam = ($$EDITBOX_RETURN << 16) OR id
				SendMessageA (GetParent(hWnd), $$WM_COMMAND, wParam, hWnd)	' send WM_COMMAND message to main window callback function
			END IF

'		CASE $$WM_CHAR :				' WM_CHAR can capture keyboard characters
'			charCode = wParam
''			PRINT "WM_CHAR message: ASCII charCode="; charCode, "CHAR="; CHR$(charCode)	' validate text entry by character

'			' allow control chars
'			IF iscntrl (charCode) THEN RETURN CallWindowProcA (#old_proc, hWnd, msg, wParam, lParam)

'			SELECT CASE charCode

'				CASE '0', '1','2','3','4','5','6','7','8','9','-','+'  : 		' check for acceptable chars
'					s$ = NULL$(128)
'					GetWindowTextA (hWnd, &s$, LEN(s$))												' get current entry in edit box
'					s$ = CSIZE$(s$) + CHR$(charCode)													' add new char
'					l = LEN(s$)
'					IF l <> 1 THEN
'						IF charCode = '-' || charCode = '+' THEN RETURN					' except - or + as first char only
'					END IF
'					RETURN CallWindowProcA (#old_proc, hWnd, msg, wParam, lParam)

'				CASE ELSE : RETURN
'			END SELECT

		CASE ELSE :
			RETURN CallWindowProcA (lpOldEditProc, hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION

FUNCTION SendPropListNotifyMessage (hWnd, curSel)

	NMPROPLISTMODIFIED nmplm

	' send a notify message to parent window
	hFrom 							= GetParent(hWnd)
	id 									= GetDlgCtrlID (hFrom)
	nmplm.hdr.hwndFrom 	= hFrom
	nmplm.hdr.idFrom   	= id
	nmplm.hdr.code     	= $$PROPLIST_SELECTION_CHANGE
	nmplm.curSel 				= curSel
	SendMessageA (GetParent(hFrom), $$WM_NOTIFY, id, &nmplm)

END FUNCTION

FUNCTION PropList_GetProps (lpProp, @name$, @curValue$, @defValue$, @itemType, @comboItems$, @dataType$, @category$, @help$)

	PROPLIST_PROPERTY plp

	IF lpProp THEN
		XstCopyMemory (lpProp, &plp, SIZE(plp))
		name$ 			= plp.propName
		curValue$ 	= plp.curValue
		defValue$ 	= plp.defValue
		itemType 		= plp.nItemType
		comboItems$ = plp.cmbItems
		dataType$ 	= plp.cDataType
		category$ 	= plp.cat
		help$ 			= plp.help
	END IF

END FUNCTION

'
'
' #########################
' #####  SetColor ()  #####
' #########################
'
FUNCTION  SetColor (txtColor, bkColor, wParam, lParam)

	SHARED hNewBrush
	DeleteObject (hNewBrush)
	hNewBrush = CreateSolidBrush(bkColor)
	SetTextColor (wParam, txtColor)
	SetBkColor (wParam, bkColor)
	RETURN hNewBrush


END FUNCTION

END PROGRAM
